# PQSign

PQSign (Post Quantum Sign) is a file signing/verifying tool use Aigis-sig algorithm.

PQSign is based on the high-performance post-quantum cryptographic algorithm library -- [PQMagic](https://gitee.com/pqcrypto/pqmagic) and [sm3sum](https://gitee.com/pqcrypto/sm3sum)。It is a [Minisign](https://jedisct1.github.io/minisign/)-like tool.

See more about PQSign and PQMagic at our website: https://pqcrypto.cn.

## Compile

```bash
git clone --recursive https://gitee.com/pqcrypto/pqsign.git
cd pqsign
```

```bash
mkdir build
cd build
cmake ..
make
```

The executable binary `pqsign` is contained in `build/bin`.

## Usage

### Generate public/secret key

```bash
./pqsign -G
```

The public key file `pqsign.pub` and secret key file `pqsign.key` are generated in the current directory.

One can further specify the generated files name by:

```bash
./pqsign -G -p mypubkey.pub -k myseckey.key
```

### Sign a file

```bash
./pqsign -S myfile.txt
```

This command will find the secret key file `pqsign.key` in the current directory to sign the given file `myfile.txt`, and generate the corresponding signature in `myfile.txt.pqsig`.

One can further specify the secret key file and output signature file's name by:

```bash
./pqsign -S myfile.txt -k myseckey.key -x mysigfile.sig
```

This command will use `myseckey.key` as secret file to sign the input file `myfile.txt`, and store the signature in `mysigfile.sig`.

### Verify a file

```bash
./pqsign -V myfile.txt
```

This command will find the public key file `pqsign.pub` in the current directory to verify the given file `myfile.txt` with corresponding signature file `myfile.txt.pqsig`.

One can specify the public key file and signature file as well by:

```bash
./pqsign -V myfile.txt -p mypubkey.pub -x mysigfile.sig
```

This command will use public key file `mypubkey.pub` to verify the input file `myfile.txt` with signature `mysigfile.sig`.

### Note

- PQsign will take a file's SM3 checksum as `Aigis-sig`'s input.
- The file's SM3 checksum is calculated before signing/verifying procedures using [sm3sum](https://gitee.com/pqcrypto/sm3sum).

## File format

### Signature file format

```txt
base64( algorithm_name || signature || signature_len )
```

Where,

- `algorithm_name`: 12 bytes long string, padding with 0 when less then 12 bytes
  - `algorithm_name` in pqsign is `Aigis-sig`
- Signature: File's signature generated by `Aigis-sig`
- signature_len: Record signature's length, it's a `size_t` data

### Public key format

```txt
base64( algorithm_name || public_key )
```

Where:

- `algorithm_name`: 12 bytes long string, padding with 0 when less then 12 bytes
  - `algorithm_name` in pqsign is `Aigis-sig`
- `public_key`: Public key generated by `Aigis-sig`

### Secret key format

```txt
base64( algorithm_name || secret_key )
```

Where:

- `algorithm_name`: 12 bytes long string, padding with 0 when less then 12 bytes
  - `algorithm_name` in pqsign is `Aigis-sig`
- `secret_key`: Secret key generated by `Aigis-sig`
